classdef PIMSoftBinary < handle
    
    % Matlab class PIMSoftBinary
    % Version 1.0   Perimed 2010
    % This class handles binary data files exported from PIMSoft.
    % Usage:
    % First declare a variable of the PIMSoftBinary class:
    % data = PIMSoftBinary;
    %
    % (for the following examples it is assumed that your variable is
    % called data)
    %
    % Then the methods in the class can be used to read binary files:
    % To open a file:
    % data.OpenFile(filename)
    % 
    % To close a file:
    % data.delete
    %
    % To get a frame of variance data:
    % data.getVarianceFrame
    %
    % To get a frame of intensity data:
    % data.getDCFrame
    %
    % The properties give information about the size of the recording:
    % data.numberOfImages
    % data.imageWidth
    % data.imageHeight
	%
	% And about the scaling factors of the used instrument:
	% data.coherenceFactor
	% data.signalGain
	%
	% data.fileVersion gives the binary file version for compatibility of future versions
    
    properties
        fileType = '';
        fileVersion = 0;
        signalGain = 0;
        coherenceFactor = 0;
        imageWidth = 0;
        imageHeight = 0;
        numberOfImages = 0;
    end
    
    properties ( Access = 'private' )
        fid = -1;
    end
    
    methods
        function OpenFile ( obj, fileName )
            if ( obj.fid ~= -1 )
                fclose ( obj.fid );
            end
            obj.fid = fopen ( fileName );
            assert ( obj.fid >= 0 );
%             fseek(obj.fid,1,'bof'); % Temporary fix (change 11 to 10 in position)
            obj.fileType = char(fread ( obj.fid, 10, 'char*1' )');
            obj.fileVersion = fread ( obj.fid, 1, 'int32' );
            calFactors = fread ( obj.fid, 2, 'double' );
            obj.signalGain = calFactors(1);
            obj.coherenceFactor = calFactors(2);
            obj.numberOfImages = fread ( obj.fid, 1, 'double' )/2;            
            data = fread ( obj.fid, 2, 'int32' );
            obj.imageWidth = data(1);
            obj.imageHeight = data(2);            
        end
        
        function delete(obj)
            disp ( 'Closing open files.' );
			if ( obj.fid ~= -1 )
				fclose(obj.fid);
			end
        end
        
        % Get one frame of variance data
        function frameData = getVarianceFrame( obj, frameNumber )            
            assert ( frameNumber > 0 && frameNumber <= obj.numberOfImages );
            frameDataPosition = 10 + 4 + 2*8 + 8 + 2*4 + ... % Offset for header
                obj.imageHeight*obj.imageWidth*(frameNumber-1)*8;
            fseek(obj.fid,frameDataPosition,'bof');
            frameData = fread( obj.fid, [obj.imageHeight obj.imageWidth], 'double' );
        end
		
        % Get one frame of intensity data
        function frameData = getDCFrame( obj, frameNumber )            
            assert ( frameNumber > 0 && frameNumber <= obj.numberOfImages );
            frameDataPosition = 10 + 4 + 2*8 + 8 + 2*4 + ... % Offset for header
                obj.imageHeight*obj.imageWidth*... 
                (obj.numberOfImages + frameNumber - 1)*8;
            fseek(obj.fid,frameDataPosition,'bof');
            frameData = fread( obj.fid, [obj.imageHeight obj.imageWidth], 'double' );
        end
    
    end
    
    
end

